=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2006-07-02 00:31:10 +0000
+++ bzrlib/conflicts.py	2006-07-02 17:35:33 +0000
@@ -274,6 +274,9 @@
             return -1
         return cmp(self._cmp_list(), other._cmp_list())
 
+    def __hash__(self):
+        return hash((type(self), self.path, self.file_id))
+
     def __eq__(self, other):
         return self.__cmp__(other) == 0
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-07-02 00:31:10 +0000
+++ bzrlib/merge.py	2006-07-02 17:35:33 +0000
@@ -390,9 +390,7 @@
             results = self.tt.apply()
             self.write_modified(results)
             try:
-                new_conflicts = (self.cooked_conflicts +
-                                 list(working_tree.conflicts()))
-                working_tree.set_conflicts(ConflictList(new_conflicts))
+                working_tree.add_conflicts(self.cooked_conflicts)
             except UnsupportedOperation:
                 pass
         finally:

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2006-07-02 00:31:10 +0000
+++ bzrlib/tests/test_merge.py	2006-07-02 17:35:33 +0000
@@ -144,5 +144,5 @@
     def test_merge_inner_conflicts(self):
         tree_a = self.make_branch_and_tree('a')
         tree_a.set_conflicts(ConflictList([TextConflict('patha')]))
-        merge_inner(tree_a.branch, tree_a, tree_a, tree_a)
+        merge_inner(tree_a.branch, tree_a, tree_a, this_tree=tree_a)
         self.assertEqual(len(tree_a.conflicts()), 1)

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-06-27 08:47:32 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-02 17:35:33 +0000
@@ -28,7 +28,7 @@
 from bzrlib.trace import mutter
 from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
                                 WorkingTree)
-from bzrlib.conflicts import ConflictList
+from bzrlib.conflicts import ConflictList, TextConflict, ContentsConflict
 
 class TestWorkingTree(TestCaseWithWorkingTree):
 
@@ -554,6 +554,28 @@
             raise TestSkipped
         self.assertEqual(tree.conflicts(), ConflictList())
 
+    def test_add_conflicts(self):
+        tree = self.make_branch_and_tree('tree')
+        try:
+            tree.add_conflicts([TextConflict('path_a')])
+        except UnsupportedOperation:
+            raise TestSkipped()
+        self.assertEqual(tree.conflicts(), 
+                         ConflictList([TextConflict('path_a')]))
+        tree.add_conflicts([TextConflict('path_a')])
+        self.assertEqual(tree.conflicts(), 
+                         ConflictList([TextConflict('path_a')]))
+        tree.add_conflicts([ContentsConflict('path_a')])
+        self.assertEqual(tree.conflicts(), 
+                         ConflictList([ContentsConflict('path_a'), 
+                                       TextConflict('path_a')]))
+        tree.add_conflicts([TextConflict('path_b')])
+        self.assertEqual(tree.conflicts(), 
+                         ConflictList([ContentsConflict('path_a'), 
+                                       TextConflict('path_a'),
+                                       TextConflict('path_b')]))
+
+
     def test_revert_clear_conflicts(self):
         tree = self.make_merge_conflicts()
         self.assertEqual(len(tree.conflicts()), 1)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-06-27 00:52:14 +0000
+++ bzrlib/workingtree.py	2006-07-02 17:35:33 +0000
@@ -1432,6 +1432,9 @@
     def set_conflicts(self, arg):
         raise UnsupportedOperation(self.set_conflicts, self)
 
+    def add_conflicts(self, arg):
+        raise UnsupportedOperation(self.add_conflicts, self)
+
     @needs_read_lock
     def conflicts(self):
         conflicts = ConflictList()
@@ -1497,6 +1500,13 @@
         self._put_rio('conflicts', conflicts.to_stanzas(), 
                       CONFLICT_HEADER_1)
 
+    @needs_write_lock
+    def add_conflicts(self, new_conflicts):
+        conflict_set = set(self.conflicts())
+        conflict_set.update(set(list(new_conflicts)))
+        self.set_conflicts(ConflictList(sorted(conflict_set,
+                                               key=Conflict.sort_key)))
+
     @needs_read_lock
     def conflicts(self):
         try:

